using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Text;

//using System.Collections;


// Important notice:
// in file "WSQImageLibraryNativeMethods32.cs"
// "namespace" must match the "namespace" of the main project
namespace wsq_demo
{
    class WSQImageLibraryNativeMethods32
    {

        static IntPtr wsq_decompressed_byte_array_ptr = IntPtr.Zero; 
        static IntPtr out_comment_text_ptr = IntPtr.Zero;
        static IntPtr wsq_compressed_byte_array_ptr = IntPtr.Zero;
        static IntPtr bitmap_ptr = IntPtr.Zero;
        IntPtr serial_number_ptr = IntPtr.Zero;

        private UTF8Encoding utf8 = new UTF8Encoding();

        public WSQImageLibraryNativeMethods32() { }
		
		
        [DllImport("WSQ_library.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        private static extern IntPtr _GenerateWSQLibrarySerialNumber();
        public IntPtr pGenerateWSQLibrarySerialNumber()
        {
            try
            {
                serial_number_ptr = _GenerateWSQLibrarySerialNumber();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "GenerateWSQLibrarySerialNumber Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return serial_number_ptr;
        }


        [DllImport("WSQ_library.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        private static extern Int32 _UnlockWSQLibrary(byte[] unlock_code);
        public Int32 pUnlockWSQLibrary(String unlock_code_string)
        {
            Int32 retVal = -1;

            try
            {
                retVal = _UnlockWSQLibrary(utf8.GetBytes(unlock_code_string));
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "UnlockWSQLibrary Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return retVal;
        }			


        [DllImport("WSQ_library.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        private static extern void _RegisterWSQ();
        public void pRegisterWSQ()
        {
			try
			{
				
				_RegisterWSQ();
				
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "RegisterWSQ Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
        }


        [DllImport("WSQ_library.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        private static extern IntPtr _CreateBMPFromFile(byte[] lpszFileName);
        public IntPtr pCreateBMPFromFile(String FileName)
        {

			if (FileName == null)
			throw new ArgumentNullException("FileName should be not null");
			if (FileName.Length == 0)
			throw new ArgumentException("Incorrect file name");
		
		    //static IntPtr bitmap_ptr = IntPtr.Zero;
		
            try
            {
            	
                bitmap_ptr = _CreateBMPFromFile(utf8.GetBytes(FileName));
                
            }
            catch (Exception ex) {
                MessageBox.Show(ex.Message, "CreateBMPFromFile Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            
            return bitmap_ptr;
            
        }


        [DllImport("WSQ_library.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        private static extern int _SaveBMPToFile(IntPtr hBitmap, byte[] filename,
            [MarshalAs(UnmanagedType.I4)] int filetype);
        public int pSaveBMPToFile(IntPtr Bitmap, String FileName, int FileType)
        {
            int retVal = -1;
			
			if (Bitmap == (IntPtr)null) 
			throw new ArgumentNullException("Bitmap should be not null");
			if (FileName == null) 
			throw new ArgumentNullException("FileName should be not null");				
			if (FileName.Length == 0)
			throw new ArgumentException("Incorrect file name");
	
            try
            {
            	
                retVal = _SaveBMPToFile(Bitmap, utf8.GetBytes(FileName), FileType);
                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "SaveBMPToFile Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            
            return retVal;
        }


        [DllImport("WSQ_library.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        
        private static extern IntPtr _CreateBMPFromWSQByteArray(
			[In] byte[] input_wsq_compressed_byte_array,
			[In] [MarshalAs(UnmanagedType.I4)] int input_wsq_compressed_byte_array_length);
			
        public IntPtr pCreateBMPFromWSQByteArray(
			[In] byte[] input_wsq_compressed_byte_array,
			[In] Int32 input_wsq_compressed_byte_array_length)
			
        {
            
			//static IntPtr bitmap_ptr = IntPtr.Zero;
			
			try
			{
				bitmap_ptr = _CreateBMPFromWSQByteArray(input_wsq_compressed_byte_array, input_wsq_compressed_byte_array_length);                                     
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "CreateBMPFromWSQByteArray Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			
			return bitmap_ptr;
			
        }



        [DllImport("WSQ_library.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        private static extern int _SaveWSQByteArrayToImageFile(
			[In] byte[] input_wsq_compressed_byte_array,
			[In] [MarshalAs(UnmanagedType.I4)] int input_wsq_compressed_byte_array_length,
			byte[] filename,
			[MarshalAs(UnmanagedType.I4)] int filetype);
			
        public int pSaveWSQByteArrayToImageFile(
			[In] byte[] input_wsq_compressed_byte_array,
			[In] Int32 input_wsq_compressed_byte_array_length,
			String FileName,
			int FileType)
			
        {
			int retVal = -1;
			
			if (FileName == null)
			throw new ArgumentNullException("FileName should be not null");
			if (FileName.Length == 0)
			throw new ArgumentException("Incorrect file name");

			try
			{
				
				retVal = _SaveWSQByteArrayToImageFile(input_wsq_compressed_byte_array, input_wsq_compressed_byte_array_length,
					utf8.GetBytes(FileName), FileType);
					                                        
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "SaveWSQByteArrayToImageFile Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			
			return retVal;
        }





        [DllImport("WSQ_library.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        private static extern void _SetShowFilePropertiesDialog(
        [MarshalAs(UnmanagedType.I4)] int file_properties_dialog);
        public void pSetShowFilePropertiesDialog(int file_properties_dialog)
        {
            try
            {
            	
                _SetShowFilePropertiesDialog(file_properties_dialog);
                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "SetShowFilePropertiesDialog Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }





        [DllImport("WSQ_library.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        
        private static extern int _WSQ_decode_stream(
            [In] byte[] input_wsq_compressed_byte_array,
            [In] [MarshalAs(UnmanagedType.I4)] int input_wsq_compressed_byte_array_length,
            out IntPtr output_decompressed_byte_array,
            [MarshalAs(UnmanagedType.I4)] out int width,
            [MarshalAs(UnmanagedType.I4)] out int height,
            [MarshalAs(UnmanagedType.I4)] out int ppi,
            out IntPtr comment_text);
            
        public int pWSQ_decode_stream(
           [In] byte[] input_wsq_compressed_byte_array,
           [In] Int32 input_wsq_compressed_byte_array_length,
           out byte[] output_decompressed_byte_array,
           out Int32 width,
           out Int32 height,
           out Int32 ppi,
           out String comment_text)
           
        {
            int retVal = -1;
			
        //static IntPtr wsq_decompressed_byte_array_ptr = IntPtr.Zero; 
        //static IntPtr out_comment_text_ptr = IntPtr.Zero;			
			
            output_decompressed_byte_array = new byte[1];
			
            width = height = ppi = 0;
            comment_text = String.Empty;
			
            try
            {
            	
                retVal = _WSQ_decode_stream(input_wsq_compressed_byte_array, input_wsq_compressed_byte_array_length,
                    out wsq_decompressed_byte_array_ptr, out width, out height,
                    out ppi, out out_comment_text_ptr);
                    
                    
                comment_text = Marshal.PtrToStringAnsi(out_comment_text_ptr);
                int lenght = width * height;
                output_decompressed_byte_array = new byte[lenght];
                Marshal.Copy(wsq_decompressed_byte_array_ptr, output_decompressed_byte_array, 0, lenght);
                
                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "WSQ_decode_stream Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            
            return retVal;
            
        }



        [DllImport("WSQ_library.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        
        private static extern int _WSQ_encode_stream(
            [In] byte[] inputBuffer,
            [In] [MarshalAs(UnmanagedType.I4)] int width,
            [In] [MarshalAs(UnmanagedType.I4)] int height,
            [In] [MarshalAs(UnmanagedType.R8)] double bitrate,
            [In] [MarshalAs(UnmanagedType.I4)] int ppi,
            [In] byte[] commentText,
            out IntPtr output_compressed_byte_array,
            [MarshalAs(UnmanagedType.I4)] out int output_compressed_byte_array_length);
            
        public int pWSQ_encode_stream(
            byte[] inputBuffer,
            int width,
            int height,
            double bitrate,
            int ppi,
            String commentText,
            out byte[] output_compressed_byte_array,
            out int output_compressed_byte_array_length)
            
        {
            int retVal = -1;
			
			//static IntPtr wsq_compressed_byte_array_ptr = IntPtr.Zero;
            
			output_compressed_byte_array = null;
			
            output_compressed_byte_array_length = 0;
			
            try
            {
            	
            	
                retVal = _WSQ_encode_stream(inputBuffer, width, height, 
                    bitrate, ppi, utf8.GetBytes(commentText),
                    out wsq_compressed_byte_array_ptr, out output_compressed_byte_array_length);
                    
                    
                if (output_compressed_byte_array_length == 0)
                    throw new ArgumentException("output_compressed_byte_array_length == 0");
				
                output_compressed_byte_array = new byte[output_compressed_byte_array_length];
                Marshal.Copy(wsq_compressed_byte_array_ptr, output_compressed_byte_array, 0, output_compressed_byte_array_length);
                
                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "WSQ_encode_stream Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            
            return retVal;
        }



        [DllImport("WSQ_library.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        private static extern IntPtr _ConvertHBITMAPtoGrayScale256(IntPtr hBitmap);
        public IntPtr pConvertHBITMAPtoGrayScale256(IntPtr Bitmap)
        {
			
			//static IntPtr bitmap_ptr = IntPtr.Zero;
			
			try
			{
				
				bitmap_ptr = _ConvertHBITMAPtoGrayScale256(Bitmap);
				
			}
			catch (Exception ex) 
			{
				MessageBox.Show(ex.Message, "ConvertHBITMAPtoGrayScale256 Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			
			return bitmap_ptr;
        }


        [DllImport("WSQ_library.dll", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl)]
        private static extern int _SaveHBITMAPtoFileAsGrayScale256BMP(IntPtr hBitmap, byte[] filename);
        public int pSaveHBITMAPtoFileAsGrayScale256BMP(IntPtr Bitmap, String FileName)
        {
			int retVal = -1;
			
			if (Bitmap == (IntPtr)null) 
			throw new ArgumentNullException("Bitmap should be not null");
			if (FileName == null) 
			throw new ArgumentNullException("FileName should be not null");				
			if (FileName.Length == 0)
			throw new ArgumentException("Incorrect file name");
			
			try
			{
				
				retVal = _SaveHBITMAPtoFileAsGrayScale256BMP(Bitmap, utf8.GetBytes(FileName));
				
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "SaveHBITMAPtoFileAsGrayScale256BMP Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			
			return retVal;
        }






    }
}
